@@ -101,9 +101,9 @@ enum
   MTYPE_OSPF_VERTEX,
   MTYPE_OSPF_NEXTHOP,
   MTYPE_OSPF_PATH,
-  MTYPE_OSPF_EXTERNAL_ROUTE,
   MTYPE_OSPF_VL_DATA,
   MTYPE_OSPF_CRYPT_KEY,
+  MTYPE_OSPF_EXTERNAL_INFO,
   MTYPE_OSPF6_TOP,
   MTYPE_OSPF6_AREA,
   MTYPE_OSPF6_IF,
