@@ -1,6 +1,6 @@
 /*
- * OSPF ASE routing.
- * Copyright (C) 1999 Alex Zinin
+ * OSPF AS External route calculation.
+ * Copyright (C) 1999, 2000 Alex Zinin, Toshiaki Takada
  *
  * This file is part of GNU Zebra.
  *
@@ -17,7 +17,8 @@
  * You should have received a copy of the GNU General Public License
  * along with GNU Zebra; see the file COPYING.  If not, write to the Free
  * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.  */
+ * 02111-1307, USA.
+ */
 
 #ifndef _ZEBRA_OSPF_ASE_H
 #define _ZEBRA_OSPF_ASE_H
@@ -29,6 +30,10 @@ struct ospf_route *ospf_find_asbr_route_through_area(struct route_table *,
 						     struct prefix_ipv4 *, 
 						     struct ospf_area *);
 
-void ospf_ase_routing (struct route_table *, struct route_table *);
+int ospf_ase_calculate (struct ospf_lsa *, struct route_table *,
+			struct route_table *);
+
+void ospf_ase_calculate_schedule ();
+void ospf_ase_calculate_timer_add ();
 
 #endif /* _ZEBRA_OSPF_ASE_H */
