@@ -1,6 +1,6 @@
 /*
  * OSPF LSDB support.
- * Copyright (C) 1999 Alex Zinin
+ * Copyright (C) 1999, 2000 Alex Zinin, Kunihiro Ishiguro, Toshiaki Takada
  *
  * This file is part of GNU Zebra.
  *
@@ -22,6 +22,7 @@
 
 #include <zebra.h>
 
+#include "thread.h"
 #include "memory.h"
 #include "hash.h"
 #include "linklist.h"
@@ -32,12 +33,12 @@
 #include "vty.h"
 #include "stream.h"
 #include "log.h"
-#include "thread.h"
 
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_interface.h"
 #include "ospfd/ospf_ism.h"
 #include "ospfd/ospf_lsa.h"
+#include "ospfd/ospf_lsdb.h"
 #include "ospfd/ospf_neighbor.h"
 #include "ospfd/ospf_nsm.h"
 #include "ospfd/ospf_flood.h"
@@ -45,9 +46,233 @@
 #include "ospfd/ospf_spf.h"
 #include "ospfd/ospf_dump.h"
 #include "ospfd/ospf_route.h"
-#include "ospfd/ospf_lsdb.h"
 
+
+void
+tmp_log ( char *str, struct ospf_lsa *lsa)
+{
+  return;
+  printf ("%s %s ", str, inet_ntoa (lsa->data->id));
+  printf ("%s\n", inet_ntoa (lsa->data->adv_router));
+}
 
+struct new_lsdb *
+new_lsdb_new ()
+{
+  struct new_lsdb *new;
+
+  new = XMALLOC (MTYPE_OSPF_LSDB, sizeof (struct new_lsdb));
+  bzero (new, sizeof (struct new_lsdb));
+  new_lsdb_init (new);
+
+  return new;
+}
+
+void
+new_lsdb_init (struct new_lsdb *lsdb)
+{
+  lsdb->type[1].db = route_table_init ();
+  lsdb->type[2].db = route_table_init ();
+  lsdb->type[3].db = route_table_init ();
+  lsdb->type[4].db = route_table_init ();
+  lsdb->type[5].db = route_table_init ();
+}
+
+void
+lsdb_prefix_set (struct prefix_ls *lp, struct ospf_lsa *lsa)
+{
+  memset (lp, 0, sizeof (struct prefix_ls));
+  lp->family = 0;
+  lp->prefixlen = 64;
+  lp->id = lsa->data->id;
+  lp->adv_router = lsa->data->adv_router;
+}
+
+/* Add new LSA to lsdb. */
+void
+new_lsdb_add (struct new_lsdb *lsdb, struct ospf_lsa *lsa)
+{
+  struct route_table *table;
+  struct prefix_ls lp;
+  struct route_node *rn;
+
+  table = lsdb->type[lsa->data->type].db;
+  lsdb_prefix_set (&lp, lsa);
+  rn = route_node_get (table, (struct prefix *)&lp);
+  if (! rn->info)
+    {
+      lsdb->type[lsa->data->type].count++;
+      lsdb->total++;
+    }
+  rn->info = lsa;
+  tmp_log ("add", lsa);
+}
+
+/* Insert an LSA to lsdb. */
+struct ospf_lsa *
+new_lsdb_insert (struct new_lsdb *lsdb, struct ospf_lsa *lsa)
+{
+  struct route_table *table;
+  struct prefix_ls lp;
+  struct route_node *rn;
+
+  table = lsdb->type[lsa->data->type].db;
+  lsdb_prefix_set (&lp, lsa);
+  rn = route_node_get (table, (struct prefix *)&lp);
+  /* Newly install LSA. */
+  if (!rn->info)
+    {
+      zlog_info ("new_lsdb_insert: Newly install");
+
+      lsdb->type[lsa->data->type].count++;
+      lsdb->total++;
+      rn->info = lsa;
+    }
+  /* Replace old LSA with new one. */
+  else
+    {
+      struct ospf_lsa *old = rn->info;
+
+      zlog_info ("new_lsdb_insert: Replace");
+
+      /* First, delete LSAs from all neighbors' retransmit-list. */
+      ospf_ls_retransmit_delete_nbr_all (lsa);
+
+      /* Second, unregister LSA from refresh_list. */
+      if (old->refresh_list)
+	ospf_refresher_unregister_lsa (old);
+
+      ospf_lsa_data_free (old->data);
+      memcpy (old, lsa, sizeof (struct ospf_lsa));
+
+      lsa->data = NULL;
+      ospf_lsa_free (lsa);
+
+      route_unlock_node (rn);
+    }
+  tmp_log ("insert", rn->info);
+  return rn->info;
+}
+
+void
+new_lsdb_delete (struct new_lsdb *lsdb, struct ospf_lsa *lsa)
+{
+  struct route_table *table;
+  struct prefix_ls lp;
+  struct route_node *rn;
+
+  table = lsdb->type[lsa->data->type].db;
+  lsdb_prefix_set (&lp, lsa);
+  rn = route_node_lookup (table, (struct prefix *) &lp);
+  if (rn)
+    {
+      rn->info = NULL;
+      route_unlock_node (rn);
+      route_unlock_node (rn);
+      lsdb->type[lsa->data->type].count--;
+      lsdb->total--;
+      tmp_log ("delete", lsa);
+      return;
+    }
+  tmp_log ("can't delete", lsa);
+}
+
+void
+new_lsdb_delete_all (struct new_lsdb *lsdb)
+{
+  struct route_table *table;
+  struct route_node *rn;
+  struct ospf_lsa *lsa;
+  int i;
+
+  for (i = OSPF_MIN_LSA; i < OSPF_MAX_LSA; i++)
+    {
+      table = lsdb->type[i].db;
+      for (rn = route_top (table); rn; rn = route_next (rn))
+	if ((lsa = (rn->info)) != NULL)
+	  {
+	    rn->info = NULL;
+	    route_unlock_node (rn);
+	    ospf_lsa_free (lsa);
+	    lsdb->type[i].count--;
+	    lsdb->total--;
+	  }
+    }
+}
+
+struct ospf_lsa *
+new_lsdb_lookup (struct new_lsdb *lsdb, struct ospf_lsa *lsa)
+{
+  struct route_table *table;
+  struct prefix_ls lp;
+  struct route_node *rn;
+  struct ospf_lsa *find;
+
+  table = lsdb->type[lsa->data->type].db;
+  lsdb_prefix_set (&lp, lsa);
+  rn = route_node_lookup (table, (struct prefix *) &lp);
+  if (rn)
+    {
+      find = rn->info;
+      route_unlock_node (rn);
+      tmp_log ("lookup", lsa);
+      return find;
+    }
+  tmp_log ("can't lookup", lsa);
+  return NULL;
+}
+
+struct ospf_lsa *
+new_lsdb_lookup_by_id (struct new_lsdb *lsdb, u_char type,
+		       struct in_addr id, struct in_addr adv_router)
+{
+  struct route_table *table;
+  struct prefix_ls lp;
+  struct route_node *rn;
+  struct ospf_lsa *find;
+
+  table = lsdb->type[type].db;
+
+  memset (&lp, 0, sizeof (struct prefix_ls));
+  lp.family = 0;
+  lp.prefixlen = 64;
+  lp.id = id;
+  lp.adv_router = adv_router;
+
+  rn = route_node_lookup (table, (struct prefix *) &lp);
+  if (rn)
+    {
+      find = rn->info;
+      route_unlock_node (rn);
+      return find;
+    }
+  return NULL;
+}
+
+unsigned long
+new_lsdb_count (struct new_lsdb *lsdb)
+{
+  return lsdb->total;
+}
+
+unsigned long
+new_lsdb_isempty (struct new_lsdb *lsdb)
+{
+  return (lsdb->total == 0);
+}
+
+void
+foreach_lsa (struct route_table *table, void *p_arg, int int_arg, 
+	     int (*callback) (struct ospf_lsa *, void *, int))
+{
+  struct route_node *rn;
+
+  for (rn = route_top (table); rn; rn = route_next (rn))
+    if (rn->info != NULL)
+      callback (rn->info, p_arg, int_arg);
+}
+
+
 void
 id_to_prefix (struct in_addr id, struct prefix *p)
 {
@@ -274,10 +499,10 @@ ospf_lsdb_add (struct ospf_lsdb *lsdb, struct ospf_lsa *new)
           lsa->data = new->data;
           ospf_ls_retransmit_delete_nbr_all (new);
 	  new->data = NULL;
-          ospf_lsa_free(new);
+          ospf_lsa_free (new);
 
           if (lsa->refresh_list)
-             ospf_refresher_unregister_lsa (lsa);
+	    ospf_refresher_unregister_lsa (lsa);
 
           zlog_info("K: ospf_lsdb_add() use %x for %x", lsa, new);
 
