@@ -94,7 +94,9 @@ connected_add_ipv4 (struct interface *ifp, struct in_addr *addr,
   if (prefix_ipv4_any (&rib))
     return;
 
-  rib_add_ipv4 (ZEBRA_ROUTE_CONNECT, 0, &rib, NULL, ifp->ifindex, 0);
+  if (if_is_up(ifp)){
+    rib_add_ipv4 (ZEBRA_ROUTE_CONNECT, 0, &rib, NULL, ifp->ifindex, 0);
+  }
 }
 
 /* Delete connected IPv4 route to the interface. */
@@ -213,7 +215,9 @@ connected_add_ipv6 (struct interface *ifp, struct in6_addr *address,
 
   connected_add (ifp, connected);
 
-  rib_add_ipv6 (ZEBRA_ROUTE_CONNECT, 0, &rib, NULL, ifp->ifindex, 0);
+  if (if_is_up(ifp)){
+    rib_add_ipv6 (ZEBRA_ROUTE_CONNECT, 0, &rib, NULL, ifp->ifindex, 0);
+  }
 }
 
 void
